# Arc PageBuilder Deployer
The Arc PageBuilder Deployer package is a simple Node script which bundles a PageBuilder repository into a `.zip` file and then makes a post request to the PageBuilder  deployer API. Once posted you can deploy your site bundle directly through the deployer window.

## Getting Started
To get started you must first install the npm package.

```bash
npm install arc-pb-deployer
```

Once installed you can then require the package and call the `deploy()` function.

```javascript
const deploy = require('arc-pb-deployer')

deploy()
```

By default the module omits a series of folders and file types that are deemed as unnecersary for a PageBuilder bundle. However if you'd like to override the regex and setup your own you can do so by passing it in as the argument.

```javascript
const deploy = require('arc-pb-deployer')

const omitted =  /node_modules$|^.*\.(scss|map|less)$|.git$|__tests__$|build$|src$|package*$|yarn.lock$|circle[.]*|bash[.]*|gulp[.]*|docker[.]*|deploy[.]*|deployment[.]*|pagebuilder[.]*|mongo[.]*|proxy[.]*/

deploy(omitted)
```

Whenever the `deploy` function runs it will generate a `.zip` file to the `deployment` folder in the root directory of the PageBuilder project, this folder should be ignored inside of the projects `.gitignore` file.

## Deploying from CircleCI
You can setup [CircleCI](https://circleci.com/) to bundle and deploy the `.zip` file automatically by adding a `deployment` section inside of the projects `.circle.yml` file. Within the example below whenever something is merged to the `master` branch it will initialize `deploy.js` which contains the code within the getting started section of this readme. The `NODE_ENV` value should correspond to the environment you want that branch to deploy to, in the exmaple below whenever the `master` branch is built it will post the bundle to the production environment.

```yml
machine:
  node:
    version: 7.9.0
  environment:
    PATH: "${PATH}:${HOME}/${CIRCLE_PROJECT_REPONAME}/node_modules/.bin"
deployment:
  production:
    branch: master
    commands:
      - NODE_ENV=production node config/deploy.js
```


### Configuation
Before you'll be able to deploy from the branches listed in the [circle.yml](circle.yml) file you'll need to setup a series of corresponding environment variables within CircleCI. To do so enter the project settings and navigate to the `Environment Variables` tab. You'll need to setup a series of variables that correspond to the environments you'd like to deploy to. For instance if there's a `dev` and `production` environment listed in the [circle.yml](circle.yml) file you'll need to setup a set of variables such as `DEV_DEPLOYER_ENDPOINT`, and `PRODUCTION_DEPLOYER_ENDPOINT`, and so on.

| Variable Name  | Description |
| ------------- | ------------- |
| `PRODUCTION_DEPLOYER_ENDPOINT`  | The API endpoint that points to the production environment, for instance `api.sandbox.demo.arcpublishing.com`.  |
| `PRODUCTION_DEPLOYER_PASSWORD` | The password for the production environment.  |
| `PRODUCTION_DEPLOYER_USERNAME` | The username for the production environment.  |
| `SLACK_NOTIFY_WEBHOOK` | The webhook path for Slack, allowing the bot to post to the correct channel when a deployment is complete.  |
| `SLACK_NOTIFY_CHANNEL` | The channel you'd like the Slack bot to post to, such as `#onboarding-channel`.  |
| `SLACK_NOTIFY_ICON` | This field is optional and allows you to override the Slack emoji avatar the bot uses. Defaults to `:arc:`.  |
| `SLACK_NOTIFY_USERNAME` | This field is optional and allows you to override the Slack username for the bot. Defaults to `PageBuilder Deployer`. |


## Developer Tools
The following developer commands are accesible after running `yarn install`.

| Command  | Description |
| ------------- | ------------- |
| `yarn lint`  | Lints the deployer package against the Airbnb styleguide.  |
